define([], function(){

    return function(query, options){
        // summary:
        //      Determines if the given activity instance matches the query object
        // query: Object
        //
        // options: dojo/store/api/Store.QueryOptions?
        //  An object that contains optional information such as sort, start, and count.
        //
        // returns: Function
        //  A function that caches the passed query under the field "matches".  See any
        //  of the "query" methods on dojo.stores.
        //

        function matches(instance) {
            if(instance.projectId && instance.contentLink) {
                return query.projectId === instance.projectId && query.contentReferences.some(function(ref){
                    return ref === instance.contentLink;
                });
            }
            else if(instance.contentLink && !instance.projectId){
                return query.contentReferences.some(function(ref){
                    return ref === instance.contentLink;
                });
            }
            return query.projectId === instance.projectId;
        }

        function execute(array) {

            // Clone the array so as not to modify the original object.
            var results = array.slice(),
                sortSet = options && options.sort;

            if(sortSet){
                results.sort(function(a, b){
                    for(var sort, i=0; sort = sortSet[i]; i++){
                        var aValue = a[sort.attribute],
                            bValue = b[sort.attribute];

                        // valueOf enables proper comparison of dates
                        aValue = aValue != null ? aValue.valueOf() : aValue;
                        bValue = bValue != null ? bValue.valueOf() : bValue;

                        if (aValue != bValue){
                            return sort.descending == (aValue == null || aValue > bValue) ? -1 : 1;
                        }
                    }
                    return 0;
                });
            }

            return results;
        }
        execute.matches = matches;

        return execute;
    };
});
