define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "epi/i18n!epi/cms/nls/episerver.cms.activities.activity",
    // Parent class and mixins
    "epi-cms/project/viewmodels/_ActivityViewModel",
    "dojo/Evented"
], function(
    declare,
    lang,
    localizations,
    // Parent class and mixins
    _ActivityViewModel,
    Evented
) {

    return declare([_ActivityViewModel, Evented], {
        // summary:
        //      View model for the activity feed. Handles setting query and store state as well as
        //      saving new message activities.
        // tags:
        //      internal

        // hideOnPost: [public] Boolean
        //      Indicates whether buttons should be hidden when a post is triggered.
        hideOnPost: false,

        // isEditEnabled: [public] Boolean
        //      Indicates whether to allow edit capabilities.
        isEditEnabled: false,

        // isSingleItemSelected: [public] Boolean
        //      Indicates whether the is only a single item selected.
        isSingleItemSelected: false,

        // readOnly: [public] Boolean
        //      Indicates whether the widget is read only.
        readOnly: false,

        postscript: function() {
            this.inherited(arguments);

            // Set the default no data message to be no query.
            this.set("noDataMessage", this.noQueryMessage);
        },

        save: function(message) {
            // summary:
            //      Saves the given message as a new message activity.
            // tags:
            //      internal
            var promise =  this.activityService.addMessage(this.selectedProjectId, this._contentLink, message);
            promise.then(this.emit.bind(this, "save"));
            promise.otherwise(lang.hitch(this, "_handleError"));
            return promise;
        },

        _selectedProjectItemsSetter: function(selectedItems) {

            selectedItems = selectedItems || [];

            // Map the selected items to a list of content references filtering out items that
            // don't have a content link, e.g. when user doesn't have access to the item.
            var references = selectedItems.filter(function(item) { return !!item.contentLink; });

            // Single selection is determined by whether there is one item selected and
            // whether the user has access to that item.
            var isSingleItem = selectedItems.length == 1 && references.length == 1;

            this.set("isSingleItemSelected", isSingleItem);

            // Set the placeholder text using the single items name.
            var placeholder = isSingleItem ?
                lang.replace(localizations.textareaplaceholder, { name: references[0].name }) : "";
            this.set("placeholderText", placeholder);

            // Set parameters needed by the add message function.
            this._contentLink = isSingleItem ? references[0].contentLink : null;

            // If there is no query then set the no data message to reflect that; otherwise
            // default to the no access message since an empty result set indicates that the
            // user has no access.
            this.set("noDataMessage", selectedItems.length ? this.noAccessMessage : this.noQueryMessage);

            // If there is no query then set the store to null so that the
            // dgrid doesn't query for everything.
            this.set("store", references.length ? this.activitiesStore : null);

            // Set the activity query if there are selected items otherwise set null.
            var query = {
                contentReferences: references.map(function(item) { return item.contentLink; }),
                projectId: this.selectedProjectId
            };
            this.set("query", references.length ? query : null);
        }
    });
});
