﻿define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/keys",
        "dojo/dom-style",
        "dojo/dom-geometry",
        "dijit/form/_FormValueWidget",
        "dijit/_WidgetsInTemplateMixin",
        "dijit/form/Textarea",
        "dijit/form/_TextBoxMixin",
        "dgrid/OnDemandList",
        "dgrid/Selection",
        "put-selector/put",
        "dijit/popup",
        "epi/shell/dgrid/selection/Extensions"
],
    function (declare,
              lang,
              keys,
              domStyle,
              domGeometry,
              _FormValueWidget,
              _WidgetsInTemplateMixin,
              TextArea,
              _TextBoxMixin,
              OnDemandList,
              Selection,
              put,
              popup,
              SelectionExtensions) {

        return declare([_FormValueWidget, _WidgetsInTemplateMixin], {
            templateString: "<div><div data-dojo-type='dijit/form/Textarea' data-dojo-attach-point='_textArea, focusNode'></div><div data-dojo-attach-point='_listNode'></div></div>",
            baseClass: "epi-autocomplete-textarea",
            value: null,
            store: null,
            placeholder: "",

            // Override _FormWidgetMixin mapping id to this.focusNode since focus node is also a widget
            // and changing its id will break widget destruction.
            _setIdAttr: "domNode",

            buildRendering: function () {
                this.inherited(arguments);

                this._textArea.set({
                    intermediateChanges: this.intermediateChanges,
                    placeholder: this.placeholder
                });

                this.own(
                    this._textArea.on("keyup", lang.hitch(this, "_onKeyUp")),
                    this._textArea.on("keydown", lang.hitch(this, "_onKeyDown")),
                    this._textArea.on("change", lang.hitch(this, "_handleOnChange")),
                    this.on("blur", lang.hitch(this, "_onBlur")),

                    this._selectList = new (declare([OnDemandList, Selection, SelectionExtensions]))({
                        store: this.store,
                        selectionMode: "single",
                        query: lang.hitch(this, "query"),
                        renderRow: function (item) {
                            return put("div", item.name);
                        }
                    }, this._listNode),
                    this._selectList.on(".dgrid-row:click", lang.hitch(this, "_onRowClicked"))
                );

                //Move the popup content off screen
                popup.moveOffScreen(this._selectList);
            },

            startup: function () {
                this.inherited(arguments);

                this._selectList.startup();
                this._selectList.refresh();
            },

            focus: function () {
                // summary:
                //		Put focus on this widget
                // remarks:
                //      Check if the widget is focusable before trying to set focus
                // tags:
                //      public overridden
                if(this.isFocusable()){
                    this.inherited(arguments);
                }
            },

            setCaretLast: function() {
                // summary:
                //      Set caret last 
                // tags:
                //      public 
                if(this._textArea.value) {
                    _TextBoxMixin.selectInputText(this._textArea.domNode, this._textArea.value.length); 
                }      
            },

            reset: function () {
                this._textArea.reset();
            },

            _setValueAttr: function (value) {
                this._set("value", value);

                // Set the new value on the textarea. Passing null as the value for
                // priorityChange causes this value to be set as the reset value if
                // one is not already set.
                this._textArea.set("value", value, null);
            },
            _getValueAttr: function () {
                return this._textArea.get("value");
            },

            _onKeyUp: function (e) {

                // If the store hasn't been defined do not trigger the popup
                if(!this.store) {
                    return;
                }

                var selectList = this._selectList,
                    textbox = this._textArea.textbox,
                    selectionStart = textbox.selectionStart;

                // open the popup when @ is pressed
                if (e.keyCode == 50 && e.altKey) {
                    this._lastAt = selectionStart - 1;

                    var contentBox = domGeometry.getContentBox(this.domNode);
                    //Set the width of the popup to be the same as the text area
                    //TODO: Perhaps it should popup where the caret is?
                    domStyle.set(selectList.domNode, "width", contentBox.w + "px");
                    popup.open({
                        parent: this,
                        popup: selectList,
                        around: this.domNode,
                        onExecute: lang.hitch(this, "_closePopup"),
                        onCancel: lang.hitch(this, "_closePopup")
                    });
                    this._isPopupOpen = true;
                }

                if (e.keyCode === keys.ESCAPE) {
                    this._closePopup();
                }

                //If the @ is erased close the popup
                var value = textbox.value;
                if ((e.keyCode === keys.DELETE || e.keyCode === keys.BACKSPACE) && this._lastAt && value[this._lastAt] !== '@') {
                    this._closePopup();
                }

                if (this._isPopupOpen) {
                    //Is handled by the key down event
                    if (e.keyCode === keys.UP_ARROW || e.keyCode === keys.DOWN_ARROW) {
                        return;
                    }

                    if (e.keyCode == keys.ENTER) {
                        //Insert the selected value
                        this._insertSelection();
                    } else {
                        //TODO: Should not do a refresh but instead send the query to the server
                        //TODO: Change the size of the popup to match the number of hits returned
                        this._queryName = value.substring(this._lastAt + 1, selectionStart).toLowerCase();
                        this._selectList.refresh()
                            .then(lang.hitch(this, function () {
                                this._selectList.moveSelectionDown();
                            }));
                    }
                }
            },

            _onKeyDown: function (e) {
                var selectList = this._selectList;

                //Emit event when enter key is hit and shift enter makes new line.
                if(e.keyCode === keys.ENTER && !e.shiftKey) {
                    this.emit("save");
                    e.preventDefault();
                }

                if (this._isPopupOpen) {
                    // Move the selection up/down in the popup
                    if (e.keyCode === keys.UP_ARROW) {
                        e.preventDefault();

                        selectList.moveSelectionUp();
                    } else if (e.keyCode === keys.DOWN_ARROW) {
                        e.preventDefault();

                        selectList.moveSelectionDown();
                    } else if (e.keyCode === keys.ENTER) {
                        e.preventDefault();
                    }
                }
            },

            _onRowClicked: function () {
                //If the user clicks on a row in the list try to insert it
                this._insertSelection();
            },

            _onBlur: function () {
                //Close the popup when we get an onBlur on the widget
                this._closePopup();
            },

            _insertSelection: function () {
                var textbox = this._textArea.textbox,
                    selectList = this._selectList,
                    selectionEnd = textbox.selectionEnd,
                    value = textbox.value;

                //Insert the selected value
                var selection = null;
                for (var id in selectList.selection) {
                    if (selectList.selection[id]) {
                        selection = selectList.row(id).data;
                        break;
                    }
                }

                if (!selection) {
                    this._closePopup();
                    return;
                }

                var selectedValue = selection.id;

                var valueToInsert = value.slice(0, this._lastAt + 1) + selectedValue;
                var newCaretPos = valueToInsert.length;
                valueToInsert += value.slice(selectionEnd);

                textbox.value = valueToInsert;

                //Set the caret position right after inserted value
                textbox.setSelectionRange(newCaretPos, newCaretPos);
                this._closePopup();
            },

            _closePopup: function () {
                this._isPopupOpen = false;
                this._lastAt = null;
                this._queryName = null;
                popup.close(this._selectList);
            },

            query: function (item) {
                //Client side filtering based on the input by the user
                if (this._queryName) {
                    return ~item.name.toLowerCase().indexOf(this._queryName) || ~item.id.toLowerCase().indexOf(this._queryName);
                }
                return true;
            }
        });
});
